#include "../include/dos.h"

/* Default command line options */
struct options opt = {
    NULL,                       /* Interface */
    NULL,                       /* Tcpdump filter */
    NULL,                       /* Log file */
    0,                          /* Default non addative logfile */
    0,                          /* Deamon mode */
    "/etc/rules",               /* File with scoring rules */
    1,                          /* Default refresh delay 1s */
    0,				/* Default timeout */
    100,                        /* Default warning score */
    300,                        /* Default critical score */
    0,                          /* Do not add timestamp to output */
	0,							/* Disable redirector */
	0,							/* N/A */
	0,							/* N/A */
	0							/* N/A */
};


/* Builtin scoring rules */
char default_rules[16][256] = {
    "quick proto tcp from any port any to any port 135 flags SYN score 40",
    "quick proto tcp from any port any to any port 139 flags SYN score 40",
    "quick proto tcp from any port any to any port 1025 flags SYN score 40",
    "quick proto tcp from any port any to any port 3306 flags SYN score 40",
    "quick proto tcp from any port any to any port 1433 flags SYN score 40",
    "quick proto tcp from any port any to any port 4000 flags SYN score 40",
    "quick proto udp from any port any to any port 1026 score 20",
    "quick proto udp from any port any to any port 1027 score 20",
    "quick proto udp from any port any to any port 1028 score 20",
    "quick proto tcp from any port any to any port any flags SYN score 1",
    "quick proto icmp from any to any score 10",
    0,
    0,
    0,
    0
};

/* hosts score table */
struct host *hosts;
struct host *tmphosts;
struct net	*netfilter;

uint32_t max_id = 1;
uint32_t level_id = 1;
uint32_t hosts_num = 0;
uint32_t packets_max = 0;
uint32_t netfilter_num = 0;
uint32_t verbose = 0;

/* rules table */
struct rule *rules;
uint8_t rules_num;

/* Mutexes */
short m_packet = 0;             /* Get packet function mutex */
short m_display = 0;            /* Display function mutex */
short m_file = 0;               /* File logging function mutex */

/* Main function */
int main(int argc, char *argv[])
{
    int t;                      /* Temporary variable */
    char errbuf[PCAP_ERRBUF_SIZE];  /* pcap error buffer */
    pcap_t *handle;             /* pcap handle */
    struct bpf_program program; /* BPF filter program */
    struct itimerval timer;     /* Timer characteristic */


	/* Init network filter */
    netfilter = calloc(1, sizeof(struct net));

    /* Parse command line arguments */
    while ((t = getopt(argc, argv, "i:f:vhl:ds:r:w:c:atP:T:C:pR:qo:")) != -1)
        switch (t) {
            /* Pcap interface */
        case 'i':
            opt.pcap_dev = optarg;
            break;

            /* External log file */
        case 'l':
            opt.logfile = optarg;
            break;

            /* Addative log file */
        case 'a':
            opt.addlogfile = 1;
            break;

            /* Pcap filter */
        case 'f':
            opt.pcap_filter = optarg;
            break;

            /* File with scoring rules */
        case 's':
            opt.rules = optarg;
            break;

            /* Enable daemon mode */
        case 'd':
            opt.daemon = 1;
            break;

            /* Report refresh delay */
        case 'r':
            opt.refresh = atoi(optarg);
            break;

            /* Score limit which cause warning */
        case 'w':
            opt.warning = atoi(optarg);
            break;

            /* Score limit which cause critical */
        case 'c':
            opt.critical = atoi(optarg);
            break;

            /* Timestamp */
        case 't':
            opt.timestamp = 1;
            break;

            /* Net filter */
        case 'C':
			{
				uint8_t 	a[4];
				uint32_t 	prefix;
				uint8_t 	p;
				char 		ip[16];
				in_addr_t	addr;

				/* Get fields of IP */
				if (sscanf(optarg, "%d.%d.%d.%d/%d", &a[0], &a[1], &a[2], &a[3], &p) != 5) {
					fprintf(stderr, "Invalid class : %s\n", optarg);
					exit(-1);
				} else {
					/* Prepare IP */
					sprintf(ip, "%d.%d.%d.%d", a[0], a[1], a[2], a[3]);
					if ((addr = inet_addr(ip)) == INADDR_NONE) {
						fprintf(stderr, "Invalid class : %s\n", optarg);
						exit(-1);
					}

					/* Prepare prefix */
					prefix = netmask(p);

					/* Prepare netfilter array and increase number of filtered networks */
					netfilter_num++;
					netfilter = Realloc(netfilter, (netfilter_num) * sizeof(struct net));

					/* Save data in netfilter array */
					netfilter[netfilter_num - 1].addr.s_addr = addr;
					netfilter[netfilter_num - 1].mask = prefix;
				}
			}

            break;

        	/* Score only hosts from private network classes */
        case 'p':
			/* Prepare netfilter array and increase number of filtered networks */
			netfilter_num += 3;
			netfilter = Realloc(netfilter, (netfilter_num) * sizeof(struct net));

			/* 10.0.0.0/8 */
			netfilter[netfilter_num - 3].addr.s_addr = inet_addr("10.0.0.0");
			netfilter[netfilter_num - 3].mask = 0xff000000;
			/* 172.16.0.0/12 */
			netfilter[netfilter_num - 2].addr.s_addr = inet_addr("172.16.0.0");
			netfilter[netfilter_num - 2].mask = 0xffe00000;
			/* 192.168.0.0/16 */
			netfilter[netfilter_num - 1].addr.s_addr = inet_addr("192.168.0.0");
			netfilter[netfilter_num - 1].mask = 0xffff0000;

            break;

            /* Maximum packets */
        case 'P':
            if (optarg != NULL && isdigit(*optarg)) {
                packets_max = atol(optarg);
                if (packets_max < 0) {
                    fprintf(stderr,
                            "Packets no. must be a digit greater than zero\n");
                    exit(1);
                }
            } else {
                fprintf(stderr,
                        "Please select valid packets number value\n");
                exit(1);
            }
            break;
			
			/* Timeout */
        case 'T':
            if (optarg != NULL && isdigit(*optarg)) {
                opt.timeout = atol(optarg);
                if (optarg[strlen(optarg) - 1] == 's')
					;
		else if (optarg[strlen(optarg) - 1] == 'm')
                    opt.timeout *= 60;  /* timeout must be in seconds */
                else if (optarg[strlen(optarg) - 1] == 'h')
                    opt.timeout *= 3600;  /* look up */
                else if (optarg[strlen(optarg) - 1] == 'd')
                    opt.timeout *= 3600 * 24;
                else if (!isdigit(optarg[strlen(optarg) -1]))
                    fprintf(stderr, "Read help about setting timeout\n");
            }
            break;

            /* Quiet mode */
        case 'q':
            opt.quiet = 1;
            break;

            /* Redirector */
        case 'R':
            opt.redir = 1;
            opt.quiet = 1;
			{	
				uint8_t 	a[4];
				char 		ip[16];
				uint16_t	p;

				in_addr_t	addr;

				/* Get fields of IP */
				if (sscanf(optarg, "%d.%d.%d.%d:%d", &a[0], &a[1], &a[2], &a[3], &p) != 5) {
					fprintf(stderr, "Invalid redirector : %s\n", optarg);
					exit(-1);
				} else {
					/* Prepare IP */
					sprintf(ip, "%d.%d.%d.%d", a[0], a[1], a[2], a[3]);
					if ((addr = inet_addr(ip)) == INADDR_NONE) {
						fprintf(stderr, "Invalid redirector : %s\n", optarg);
						exit(-1);
					}
				}

				strncpy((void *)&opt.rhost, (void *)ip, strlen(ip));
				opt.rport = p;

#ifdef DEBUG
				fprintf(stderr, "%s : %d", opt.rhost, opt.rport);
#endif
			}
			
            break;

            /* Redirector output format */
        case 'o':
				if (!strncmp(optarg,"ipnat", strlen(optarg)))
					opt.rtype = REDIR_IPNAT;
				else {
					fprintf(stderr, "Invalid redirector output format.\n");
					exit(-1);
				}
			break;

            /* Print version */
        case 'v':
	    verbose++;
	    break;

            /* Help */
        case 'h':
            usage();
            exit(2);
            break;

            /* Default action */
        default:
            usage();
            break;
        }

    if (verbose>0)
    /* Print version on top */
	version();

    if (getuid()) {
        fprintf(stderr,
                "You must be root to run this application\nQuitting...\n");
        exit(-1);
    }

    /* Checks */
    if ((opt.daemon == 1) && (opt.logfile == NULL)) {
        fprintf(stderr, "Daemon mode require log file !\n");
        exit(1);
    }

    /* Daemonize */
    if (opt.daemon)
        if (daemon(0, 0) != 0) {
            perror("Error entering daemon mode: ");
            exit(-1);
        }

    /* Init tables */
    hosts = calloc(2, sizeof(struct host));
    rules = calloc(1, sizeof(struct rule));
    rules_num = 0;
    bzero(hosts, sizeof(hosts));
    bzero(rules, sizeof(rules));

    /* Init rules table */
    if (init_rules(opt.rules) != 0) {
        fprintf(stderr, "Could not init scoring rules !\n");
        exit(-1);
    }
#ifdef DEBUG
    fprintf(stderr, "Loaded %d active rules...\n", rules_num);
#endif

    /* Init BPF */
    /* Set up interface */
    if ((!opt.pcap_dev)
        && ((opt.pcap_dev = pcap_lookupdev(errbuf)) == NULL)) {
        fprintf(stderr, "%s\n", errbuf);
        exit(-1);
    }

    if ((handle = pcap_open_live(opt.pcap_dev, 68, 0, 0, errbuf)) == NULL) {
        fprintf(stderr, "%s\n", errbuf);
        exit(-1);
    }

    /* Add bpf filter */
    if (opt.pcap_filter) {
        if (pcap_compile(handle, &program, opt.pcap_filter, 0, 0) == -1) {
            fprintf(stderr, "Error - `pcap_compile()'\n");
            return 1;
        }

        if (pcap_setfilter(handle, &program) == -1) {
            fprintf(stderr, "Error - `pcap_setfilter()'\n");
            return 1;
        }

        pcap_freecode(&program);
    }

    /* Set timer to show statistics every delay timeout */
    bzero(&timer, sizeof(timer));
    timer.it_interval.tv_sec = 1;
    timer.it_value.tv_sec = 1;

    if (setitimer(ITIMER_REAL, &timer, NULL) != 0) {
        perror("Timer");
        exit(5);
    }

    if (signal(SIGALRM, display) == SIG_ERR) {
        perror("Signal");
        exit(6);
    }

    /* Main loop */
    printf("Starting capturing engine on %s...\n", opt.pcap_dev);
    pcap_loop(handle, -1, got_packet, NULL);

    /* Exit program */
    printf("Closing capturing engine...\n");
    pcap_close(handle);
}
